@@ -26,6 +26,7 @@
 #include "enumliteral.h"
 #include "entityattribute.h"
 #include "model_utils.h"
+#include "notewidget.h"
 #include "umldoc.h"
 #include "umllistview.h"
 #include "umllistviewitem.h"
@@ -178,7 +179,7 @@ bool UMLClipboard::paste(const QMimeData* data)
 
     int codingType = UMLDragData::getCodingType(data);
 
-    QString mimeType = "application/x-uml-clip" + QString::number(codingType);
+    QString mimeType = QLatin1String("application/x-uml-clip") + QString::number(codingType);
     uDebug() << "Pasting mimeType=" << mimeType << "data=" << data->data(mimeType);
 
     bool result = false;
@@ -404,12 +405,17 @@ bool UMLClipboard::pasteClip2(const QMimeData* data)
     UMLViewList         views;
 
     if (!UMLDragData::decodeClip2(data, objects, views)) {
+        uDebug() << "UMLDragData::decodeClip2 returned error";
         return false;
     }
 
-    foreach (UMLView* pView, views) {
-        if(!doc->addUMLView(pView)) {
-            return false;
+    if (NoteWidget::s_pCurrentNote) {
+        NoteWidget::s_pCurrentNote = NULL;
+    } else {
+        foreach (UMLView* pView, views) {
+            if (!doc->addUMLView(pView)) {
+                return false;
+            }
         }
     }
 
@@ -586,7 +592,7 @@ bool UMLClipboard::pasteClip5(const QMimeData* data)
                 if (parent->addAttribute(att, idchanges)) {
                     result = true;
                 } else {
-                    uError() << "" << parent->name() << "->addAttribute("
+                    uError() << parent->name() << "->addAttribute("
                              << att->name() << ") failed";
                 }
                 break;
@@ -602,7 +608,7 @@ bool UMLClipboard::pasteClip5(const QMimeData* data)
                 if (parent->addOperation(op, idchanges)) {
                     result = true;
                 } else {
-                    uError() << "" << parent->name() << "->addOperation("
+                    uError() << parent->name() << "->addOperation("
                              << op->name() << ") failed";
                 }
                 break;
@@ -618,7 +624,7 @@ bool UMLClipboard::pasteClip5(const QMimeData* data)
                 if (parent->addTemplate(tp, idchanges)) {
                     result = true;
                 } else {
-                    uError() << "" << parent->name() << "->addTemplate("
+                    uError() << parent->name() << "->addTemplate("
                              << tp->name() << ") failed";
                 }
                 break;
@@ -643,7 +649,7 @@ bool UMLClipboard::pasteClip5(const QMimeData* data)
                if (enumParent->addEnumLiteral(enl, idchanges)) {
                    result = true;
                } else {
-                   uError() << "" << enumParent->name() << "->addEnumLiteral("
+                   uError() << enumParent->name() << "->addEnumLiteral("
                             << enl->name() << ") failed";
                }
                break;
@@ -667,7 +673,7 @@ bool UMLClipboard::pasteClip5(const QMimeData* data)
                 if (entityParent->addEntityAttribute(att, idchanges)) {
                     result = true;
                 } else {
-                    uError() << "" << parent->name() << "->addEntityAttribute(" << att->name() << ") failed";
+                    uError() << parent->name() << "->addEntityAttribute(" << att->name() << ") failed";
                 }
                 break;
             }
