@@ -18,6 +18,7 @@ class QGridLayout;
 class QLabel;
 class UMLWidget;
 class KLineEdit;
+class KComboBox;
 
 /**
  * Dialog utilities.
@@ -26,7 +27,7 @@ class KLineEdit;
  */
 namespace Dialog_Utils {
 
-KLineEdit* makeLabeledEditField(QGroupBox *containingBox, QGridLayout *layout, int row,
+KLineEdit* makeLabeledEditField(QGridLayout *layout, int row,
                                 QLabel * &label, const QString& labelText,
                                 KLineEdit * &editField,
                                 const QString& editFieldText = QString());
@@ -34,6 +35,7 @@ KLineEdit* makeLabeledEditField(QGroupBox *containingBox, QGridLayout *layout, i
 void askNameForWidget(UMLWidget * &targetWidget, const QString& dialogTitle,
                       const QString& dialogPrompt, const QString& defaultName);
 
+void insertStereotypesSorted(KComboBox *kcb, const QString& type);
 
 }
 
