@@ -38,7 +38,7 @@ protected:
 
     bool apply();
 
-    void insertTypesSorted(const QString& type = "");
+    void insertTypesSorted(const QString& type = QString());
 
     void insertAttribute(const QString& type, int index = -1);
 
