@@ -48,7 +48,7 @@ protected:
 
     bool apply();
 
-    void insertTypesSorted(const QString& type = "");
+    void insertTypesSorted(const QString& type = QString());
     void insertStereotypesSorted(const QString& type);
 
     UMLOperation*  m_operation;  ///< The operation to represent.
