@@ -55,7 +55,7 @@ DocWindow::DocWindow(UMLDoc * doc, QWidget *parent)
     m_modifiedLabel->setToolTip(i18n("Flag whether documentation was modified"));
     statusLayout->addWidget(m_modifiedLabel, 0, 5, 1, 1);
     m_docTE = new KTextEdit(this);
-    m_docTE->setText("");
+    m_docTE->setText(QString());
     //m_docTE->setWordWrapMode(QTextEdit::WidgetWidth);
     QVBoxLayout* docLayout = new QVBoxLayout(this);
     docLayout->addLayout(statusLayout);
@@ -314,7 +314,7 @@ bool DocWindow::isModified()
 
 /**
  * An association was removed from the UMLScene.
- * If the association removed was the association which documentation is
+ * If the association removed was the association whose documentation is
  * being shown, m_pAssocWidget is set to 0.
  */
 void DocWindow::slotAssociationRemoved(AssociationWidget* association)
