@@ -27,6 +27,8 @@
 #include "model_utils.h"
 #include "objectnodewidget.h"
 #include "objectwidget.h"
+#include "notewidget.h"
+#include "pinportbase.h"
 #include "preconditionwidget.h"
 #include "signalwidget.h"
 #include "statewidget.h"
@@ -34,6 +36,8 @@
 #include "umldoc.h"
 #include "umlscene.h"
 #include "umlview.h"
+#include "umllistview.h"
+#include "umllistviewitem.h"
 #include "widget_utils.h"
 #include "widgetbase.h"
 
@@ -53,7 +57,7 @@ const bool CHECKABLE = true;
  * @param view     The UMLView object
  */
 ListPopupMenu::ListPopupMenu(QWidget *parent, MenuType type, UMLView * view)
-  : KMenu(parent)
+  : KMenu(parent), m_isListView(false)
 {
     m_TriggerObject.m_View = view;
     m_TriggerObjectType = tot_View;
@@ -66,10 +70,10 @@ ListPopupMenu::ListPopupMenu(QWidget *parent, MenuType type, UMLView * view)
  *
  * @param parent   The parent to ListPopupMenu.
  * @param type     The type of menu to display.
- * @param object   The WidgetBase object.
+ * @param widget   The WidgetBase object.
  */
 ListPopupMenu::ListPopupMenu(QWidget *parent, MenuType type, WidgetBase *widget)
-  : KMenu(parent)
+  : KMenu(parent), m_isListView(false)
 {
     m_TriggerObject.m_Widget = widget;
     m_TriggerObjectType = tot_Widget;
@@ -85,7 +89,7 @@ ListPopupMenu::ListPopupMenu(QWidget *parent, MenuType type, WidgetBase *widget)
  * @param object   The UMLObject of the ListViewItem
  */
 ListPopupMenu::ListPopupMenu(QWidget *parent, UMLListViewItem::ListViewType type, UMLObject* object)
-  : KMenu(parent)
+  : KMenu(parent), m_isListView(true)
 {
     m_TriggerObject.m_Object = object;
     m_TriggerObjectType = tot_Object;
@@ -191,6 +195,10 @@ ListPopupMenu::ListPopupMenu(QWidget *parent, UMLListViewItem::ListViewType type
         mt = mt_Component;
         break;
 
+    case UMLListViewItem::lvt_Port:
+        mt = mt_Port;
+        break;
+
     case UMLListViewItem::lvt_Node:
         mt = mt_Node;
         break;
@@ -298,7 +306,13 @@ ListPopupMenu::ListPopupMenu(QWidget * parent, WidgetBase * object, bool multi,
     bool bCutState = UMLApp::app()->isCutCopyState();
     setActionEnabled(mt_Cut, bCutState);
     setActionEnabled(mt_Copy, bCutState);
-    setActionEnabled(mt_Paste, false);
+    bool pasteAvailable = false;
+    if (object->baseType() == WidgetBase::wt_Note &&
+            UMLApp::app()->listView()->startedCopy()) {
+        NoteWidget::s_pCurrentNote = static_cast<NoteWidget*>(object);
+        pasteAvailable = true;
+    }
+    setActionEnabled(mt_Paste, pasteAvailable);
     setupActionsData();
 }
 
@@ -378,6 +392,19 @@ void ListPopupMenu::insertSingleSelectionMenu(WidgetBase* object)
         insert(mt_Properties);
         break;
 
+    case WidgetBase::wt_Port:
+        insertSubMenuColor(object->useFillColor());
+        insertStdItems(false);
+        insert(mt_NameAsTooltip, i18n("Name as Tooltip"), CHECKABLE);
+        {
+            PinPortBase *pW = static_cast<PinPortBase*>(object);
+            FloatingTextWidget *ft = pW->floatingTextWidget();
+            if (ft == NULL)
+                m_actions[mt_NameAsTooltip]->setChecked(true);
+        }
+        insert(mt_Properties);
+        break;
+
     case WidgetBase::wt_Object:
         {
             //Used for sequence diagram and collaboration diagram widgets
@@ -516,6 +543,13 @@ void ListPopupMenu::insertSingleSelectionMenu(WidgetBase* object)
         insert(mt_Clear, Icon_Utils::SmallIcon(Icon_Utils::it_Clear), i18nc("clear precondition", "Clear"));
         addSeparator();
         insert(mt_Rename, i18n("Change Text..."));
+        if (type == WidgetBase::wt_Pin) {
+            insert(mt_NameAsTooltip, i18n("Name as Tooltip"), CHECKABLE);
+            PinPortBase *pW = static_cast<PinPortBase*>(object);
+            FloatingTextWidget *ft = pW->floatingTextWidget();
+            if (ft == NULL)
+                m_actions[mt_NameAsTooltip]->setChecked(true);
+        }
         insert(mt_Delete);
         insert(mt_Change_Font);
         break;
@@ -635,6 +669,9 @@ void ListPopupMenu::insert(MenuType m)
     case mt_Rename:
         m_actions[m] = addAction(Icon_Utils::SmallIcon(Icon_Utils::it_Rename), i18n("Rename..."));
         break;
+    case mt_Show:
+        m_actions[m] = addAction(Icon_Utils::SmallIcon(Icon_Utils::it_Show), i18n("Show"));
+        break;
     case mt_Delete:
         m_actions[m] = addAction(Icon_Utils::SmallIcon(Icon_Utils::it_Delete), i18n("Delete"));
         break;
@@ -720,12 +757,15 @@ void ListPopupMenu::insert(const MenuType m, KMenu* menu)
     case mt_Component:
         m_actions[m] = menu->addAction(Icon_Utils::SmallIcon(Icon_Utils::it_Component), i18n("Component"));
         break;
+    case mt_Port:
+        m_actions[m] = menu->addAction(Icon_Utils::SmallIcon(Icon_Utils::it_Port), i18n("Port"));
+        break;
     case mt_Artifact:
         m_actions[m] = menu->addAction(Icon_Utils::SmallIcon(Icon_Utils::it_Artifact), i18n("Artifact"));
         break;
     case mt_Component_Diagram:
         {
-            QAction* act = UMLApp::app()->actionCollection()->action("new_component_diagram");
+            QAction* act = UMLApp::app()->actionCollection()->action(QLatin1String("new_component_diagram"));
             //don't keep a local copy of pointer which resides somewhere else (in this case - in actionCollection())
             //m_actions[m] = act;
             menu->addAction(act);
@@ -736,7 +776,7 @@ void ListPopupMenu::insert(const MenuType m, KMenu* menu)
         break;
     case mt_Deployment_Diagram:
         {
-            QAction* act = UMLApp::app()->actionCollection()->action("new_deployment_diagram");
+            QAction* act = UMLApp::app()->actionCollection()->action(QLatin1String("new_deployment_diagram"));
             //m_actions[m] = act;
             menu->addAction(act);
         }
@@ -752,7 +792,7 @@ void ListPopupMenu::insert(const MenuType m, KMenu* menu)
         break;
     case mt_EntityRelationship_Diagram:
         {
-            QAction* act = UMLApp::app()->actionCollection()->action("new_entityrelationship_diagram");
+            QAction* act = UMLApp::app()->actionCollection()->action(QLatin1String("new_entityrelationship_diagram"));
             //m_actions[m] = act;
             menu->addAction(act);
         }
@@ -768,7 +808,7 @@ void ListPopupMenu::insert(const MenuType m, KMenu* menu)
         break;
     case mt_UseCase_Diagram:
         {
-            QAction* act = UMLApp::app()->actionCollection()->action("new_use_case_diagram");
+            QAction* act = UMLApp::app()->actionCollection()->action(QLatin1String("new_use_case_diagram"));
             //m_actions[m] = act;
             menu->addAction(act);
         }
@@ -843,7 +883,7 @@ void ListPopupMenu::insert(const MenuType m, KMenu* menu, const QString & text,
 }
 
 /**
- * Shortcut for the frequently used insertStdItem() calls.
+ * Shortcut for the frequently used insert() calls.
  *
  * @param insertLeadingSeparator   Set this true if the group shall
  *                                 start with a separator.
@@ -865,7 +905,8 @@ void ListPopupMenu::insertStdItems(bool insertLeadingSeparator /* = true */,
     else if (Model_Utils::isCloneable(type))
         insert(mt_Clone);
     insert(mt_Delete);
-    insert(mt_Resize, i18n("Resize"));
+    if (!m_isListView)
+        insert(mt_Resize, i18n("Resize"));
 }
 
 /**
@@ -887,11 +928,11 @@ void ListPopupMenu::insertContainerItems(bool folderAndDiagrams)
     insert(mt_Enum, menu, Icon_Utils::SmallIcon(Icon_Utils::it_Enum), i18n("Enum"));
     insert(mt_Package, menu, Icon_Utils::SmallIcon(Icon_Utils::it_Package), i18n("Package"));
     if (folderAndDiagrams) {
-        menu->addAction(UMLApp::app()->actionCollection()->action("new_class_diagram"));
-        menu->addAction(UMLApp::app()->actionCollection()->action("new_sequence_diagram"));
-        menu->addAction(UMLApp::app()->actionCollection()->action("new_collaboration_diagram"));
-        menu->addAction(UMLApp::app()->actionCollection()->action("new_state_diagram"));
-        menu->addAction(UMLApp::app()->actionCollection()->action("new_activity_diagram"));
+        menu->addAction(UMLApp::app()->actionCollection()->action(QLatin1String("new_class_diagram")));
+        menu->addAction(UMLApp::app()->actionCollection()->action(QLatin1String("new_sequence_diagram")));
+        menu->addAction(UMLApp::app()->actionCollection()->action(QLatin1String("new_collaboration_diagram")));
+        menu->addAction(UMLApp::app()->actionCollection()->action(QLatin1String("new_state_diagram")));
+        menu->addAction(UMLApp::app()->actionCollection()->action(QLatin1String("new_activity_diagram")));
     }
     addMenu(menu);
 }
@@ -1032,7 +1073,11 @@ void ListPopupMenu::insertSubmodelAction()
 void ListPopupMenu::insertLayoutItems(UMLView *view)
 {
     QList<MenuType> types;
-    types << mt_Apply_Layout << mt_Apply_Layout1 << mt_Apply_Layout2 << mt_Apply_Layout3 << mt_Apply_Layout4 << mt_Apply_Layout5 << mt_Apply_Layout6 << mt_Apply_Layout7 << mt_Apply_Layout8 << mt_Apply_Layout9;
+    types << mt_Apply_Layout  << mt_Apply_Layout1
+          << mt_Apply_Layout2 << mt_Apply_Layout3
+          << mt_Apply_Layout4 << mt_Apply_Layout5
+          << mt_Apply_Layout6 << mt_Apply_Layout7
+          << mt_Apply_Layout8 << mt_Apply_Layout9;
     LayoutGenerator generator;
     if (generator.isEnabled()) {
         QHash<QString, QString> configFiles;
@@ -1041,7 +1086,8 @@ void ListPopupMenu::insertLayoutItems(UMLView *view)
             foreach(const QString &key, configFiles.keys()) {
                 if (i >= types.size())
                     break;
-                if (key == "export" && !Settings::optionState().autoLayoutState.showExportLayout)
+                if (key == QLatin1String("export") &&
+                        !Settings::optionState().autoLayoutState.showExportLayout)
                     continue;
                 insert(types[i], QPixmap(), i18n("apply '%1'", configFiles[key]));
                 QAction* action = getAction(types[i]);
@@ -1323,6 +1369,8 @@ void ListPopupMenu::insertSubMenuNew(MenuType type)
             break;
         case mt_Component:
             insert(mt_Component, menu);
+            if (Settings::optionState().generalState.uml2)
+                insert(mt_Port, menu);
             insert(mt_Artifact, menu);
             break;
         case mt_Component_View:
@@ -1384,6 +1432,8 @@ void ListPopupMenu::insertSubMenuNew(MenuType type)
         case mt_On_Component_Diagram:
             insert(mt_Subsystem, menu, Icon_Utils::SmallIcon(Icon_Utils::it_Subsystem), i18n("Subsystem..."));
             insert(mt_Component, menu, Icon_Utils::SmallIcon(Icon_Utils::it_Component), i18n("Component..."));
+            if (Settings::optionState().generalState.uml2)
+                insert(mt_Port, menu, Icon_Utils::SmallIcon(Icon_Utils::it_Port), i18n("Port..."));
             insert(mt_Artifact, menu, Icon_Utils::SmallIcon(Icon_Utils::it_Artifact), i18n("Artifact..."));
             break;
         case mt_On_Deployment_Diagram:
@@ -1637,6 +1687,8 @@ void ListPopupMenu::setupMenu(MenuType type)
     case mt_Class:
         insertSubMenuNew(type);
         insertStdItems();
+        if (m_isListView)
+            insert(mt_Show);
         addSeparator();
         insert(mt_Import_Class);
         addSeparator();
@@ -1646,12 +1698,16 @@ void ListPopupMenu::setupMenu(MenuType type)
     case mt_Interface:
         insertSubMenuNew(type);
         insertStdItems();
+        if (m_isListView)
+            insert(mt_Show);
         insert(mt_Properties);
         break;
 
     case mt_Package:
         insertContainerItems(false);
         insertStdItems();
+        if (m_isListView)
+            insert(mt_Show);
         insert(mt_Properties);
         addSeparator();
         insert(mt_Expand_All);
@@ -1690,6 +1746,7 @@ void ListPopupMenu::setupMenu(MenuType type)
 
     case mt_Datatype:
     case mt_EnumLiteral:
+    case mt_Port:
     case mt_Node:
     case mt_Artifact:
     case mt_Actor:
