@@ -26,7 +26,7 @@ public:
         level++;
         *this << className
               << "(\n"
-              << QString().fill('.', level).toAscii().constData()
+              << QString().fill(QLatin1Char('.'), level).toAscii().constData()
               ;
     }
 
@@ -34,7 +34,7 @@ public:
     {
         --level;
         *this << "\n"
-              << QString().fill(',', level).toAscii().constData()
+              << QString().fill(QLatin1Char(','), level).toAscii().constData()
               << ")";
               ;
     }
@@ -97,6 +97,23 @@ QString PetalNode::name() const
     return m_initialArgs.first();
 }
 
+/**
+ * Return the documentation from a petal node with carriage
+ * return handling and removed surrounding quotation marks
+ * if present.
+ */
+QString PetalNode::documentation() const
+{
+    QString s = findAttribute(QLatin1String("documentation")).string.trimmed();
+    if (s.isEmpty())
+        return s;
+    s.replace(QLatin1String("\\n"), QLatin1String("\n"));
+    if (s.startsWith(QLatin1Char('\"')) && s.endsWith(QLatin1Char('\"')))
+        return s.mid(1 ,s.length()-2);
+    else
+        return s;
+}
+
 PetalNode::NameValueList PetalNode::attributes() const
 {
     return m_attributes;
