@@ -28,7 +28,7 @@ class UMLDoc;
 class UMLListView;
 class UMLView;
 class WorkToolBar;
-class SettingsDlg;
+class SettingsDialog;
 class UMLViewImageExporterAll;
 class RefactoringAssistant;
 class XhtmlGenerator;
@@ -60,6 +60,7 @@ class QPushButton;
 class QLabel;
 class QListWidget;
 class QSlider;
+class UMLAppPrivate;
 
 /**
  * The base class for UML application windows. It sets up the main
@@ -99,6 +100,7 @@ public:
     DocWindow * docWindow() const;
     QListWidget *logWindow() const;
 
+    QCursor defaultCursor();
 
     void setModified(bool _m);
 
@@ -133,7 +135,7 @@ public:
 
     QWidget* mainViewWidget();
 
-    void setCurrentView(UMLView* view);
+    void setCurrentView(UMLView* view, bool updateTreeView = true);
     UMLView* currentView() const;
 
     void setImageMimeType(const QString& mimeType);
@@ -187,7 +189,6 @@ protected:
     void initView();
 
     virtual bool queryClose();
-    virtual bool queryExit();
 
     virtual void saveProperties(KConfigGroup & cfg);
     virtual void readProperties(const KConfigGroup & cfg);
@@ -211,6 +212,9 @@ public slots:
     void slotFileQuit();
     void slotFileExportDocbook();
     void slotFileExportXhtml();
+    void slotFind();
+    void slotFindNext();
+    void slotFindPrevious();
     void slotEditCut();
     void slotEditCopy();
     void slotEditPaste();
@@ -305,6 +309,7 @@ private slots:
 
 private:
     static UMLApp* s_instance;  ///< The last created instance of this class.
+    UMLAppPrivate *m_d;
 
     QMenu* findMenu(const QString &name);
 
@@ -313,7 +318,7 @@ private:
 
     void resetStatusMsg();
 
-    void setProgLangAction(Uml::ProgrammingLanguage::Enum pl, const QString& name, const QString& action);
+    void setProgLangAction(Uml::ProgrammingLanguage::Enum pl, const char* name, const char* action);
 
     static bool canDecode(const QMimeData* mimeSource);
 
@@ -361,7 +366,6 @@ private:
     QDockWidget* m_logDock;            ///< Contains the log window widget.
 
     DocWindow*   m_docWindow;         ///< Documentation window.
-    QListWidget* m_logWindow;         ///< Logging window.
     QUndoView*   m_pQUndoView;         ///< Undo / Redo Viewer
     RefactoringAssistant* m_refactoringAssist;  ///< Refactoring assistant.
 
@@ -424,7 +428,7 @@ private:
 
     QString m_imageMimeType;  ///< Default mime type to use for image export.
 
-    SettingsDlg* m_settingsDlg;  ///< The global UML settings dialog.
+    SettingsDialog* m_settingsDialog;  ///< The global UML settings dialog.
 
     UMLViewImageExporterAll* m_imageExporterAll;  ///< Used to export all the views.
 
