@@ -45,10 +45,11 @@
 #include "umlviewimageexporter.h"
 #include "usecase.h"
 #include "model_utils.h"
+#include "optionstate.h"
 #include "uniqueid.h"
 #include "idchangelog.h"
 #include "umldragdata.h"
-#include "classpropdlg.h"
+#include "classpropertiesdialog.h"
 #include "umlattributedialog.h"
 #include "umlentityattributedialog.h"
 #include "umloperationdialog.h"
@@ -64,6 +65,7 @@
 #include <kmessagebox.h>
 #include <kinputdialog.h>
 #include <kapplication.h>
+#include <ktabwidget.h>
 
 // qt includes
 #include <QDropEvent>
@@ -307,8 +309,9 @@ void UMLListView::keyPressEvent(QKeyEvent *ke)
 /**
  * Called when a right mouse button menu has an item selected.
  * @param action   the selected action
+ * @param position the position of the menu on the diagram (only used for multi selection "Show")
  */
-void UMLListView::slotMenuSelection(QAction* action)
+void UMLListView::slotMenuSelection(QAction* action, const QPoint &position)
 {
     UMLListViewItem * currItem = static_cast<UMLListViewItem*>(currentItem());
     if (!currItem) {
@@ -337,6 +340,11 @@ void UMLListView::slotMenuSelection(QAction* action)
         addNewItem(currItem, UMLListViewItem::lvt_Component);
         break;
 
+    case ListPopupMenu::mt_Port:
+        if (Settings::optionState().generalState.uml2)
+            addNewItem(currItem, UMLListViewItem::lvt_Port);
+        break;
+
     case ListPopupMenu::mt_Node:
         addNewItem(currItem, UMLListViewItem::lvt_Node);
         break;
@@ -460,13 +468,13 @@ void UMLListView::slotMenuSelection(QAction* action)
             }
             // configure & show the file dialog
             const QString rootDir(m_doc->url().directory());
-            QPointer<KFileDialog> fileDialog = new KFileDialog(rootDir, "*.xml", this);
+            QPointer<KFileDialog> fileDialog = new KFileDialog(rootDir, QLatin1String("*.xml"), this);
             fileDialog->setCaption(i18n("Externalize Folder"));
             fileDialog->setOperationMode(KFileDialog::Other);
             // set a sensible default filename
             QString defaultFilename = current->text(0).toLower();
-            defaultFilename.replace(QRegExp("\\W+"), "_");
-            defaultFilename.append(".xml");  // default extension
+            defaultFilename.replace(QRegExp(QLatin1String("\\W+")), QLatin1String("_"));
+            defaultFilename.append(QLatin1String(".xml"));  // default extension
             fileDialog->setSelection(defaultFilename);
             KUrl selURL;
             if (fileDialog->exec() == QDialog::Accepted) {
@@ -507,8 +515,8 @@ void UMLListView::slotMenuSelection(QAction* action)
             modelFolder->setFolderFile(fileName);
             // Recompute text of the folder
             QString folderText = current->text(0);
-            folderText.remove(QRegExp("\\s*\\(.*$"));
-            folderText.append(" (" + fileName + ')');
+            folderText.remove(QRegExp(QLatin1String("\\s*\\(.*$")));
+            folderText.append(QLatin1String(" (") + fileName + QLatin1Char(')'));
             current->setText(folderText);
             break;
         }
@@ -524,7 +532,7 @@ void UMLListView::slotMenuSelection(QAction* action)
             modelFolder->setFolderFile(QString());
             // Recompute text of the folder
             QString folderText = current->text(0);
-            folderText.remove(QRegExp("\\s*\\(.*$"));
+            folderText.remove(QRegExp(QLatin1String("\\s*\\(.*$")));
             current->setText(folderText);
             break;
         }
@@ -550,6 +558,63 @@ void UMLListView::slotMenuSelection(QAction* action)
         deleteItem(currItem);
         break;
 
+    case ListPopupMenu::mt_Show:
+        // first check if we are on a diagram
+        if (!Model_Utils::typeIsDiagram(lvt)) {
+            UMLObject * object = currItem->umlObject();
+            if (!object) {
+                uError() << "UMLObjet of ... is null! Doing nothing.";
+                return;
+            }
+            QList<UMLWidget*> findResults;
+
+            if (Model_Utils::typeIsCanvasWidget(lvt)) {
+                UMLViewList views = m_doc->viewIterator();
+                foreach (UMLView *view, views) {
+                    foreach(UMLWidget *widget, view->umlScene()->widgetList()) {
+                        if (object == widget->umlObject()) {
+                            findResults.append(widget);
+                        }
+                    }
+                }
+            }
+
+            if (findResults.size() == 0)  {
+                break;
+            }
+
+            UMLWidget *selectedResult = 0;
+
+            if (findResults.size() > 1) {
+                QMenu menu(this);
+                int i = 0;
+                foreach(UMLWidget *w, findResults) {
+                    QAction *action = menu.addAction(w->umlScene()->name() + QLatin1Char(':') + w->name());
+                    action->setData(i++);
+                }
+                QAction *action = menu.exec(position);
+                if (action) {
+                    selectedResult = findResults.at(action->data().toInt());
+                }
+            } else {
+                selectedResult = findResults.first();
+            }
+
+            if (!selectedResult) {
+                break;
+            }
+
+            UMLView *view = selectedResult->umlScene()->activeView();
+            selectedResult->umlScene()->setIsOpen(true);
+            view->setZoom(100);
+            if (UMLApp::app()->currentView() != view) {
+                UMLApp::app()->setCurrentView(view, false);
+            }
+            view->centerOn(selectedResult->pos());
+            selectedResult->setSelected(true);
+        }
+        break;
+
     case ListPopupMenu::mt_Properties:
         // first check if we are on a diagram
         if (Model_Utils::typeIsDiagram(lvt)) {
@@ -571,7 +636,7 @@ void UMLListView::slotMenuSelection(QAction* action)
             umlType = object->baseType();
 
             if (Model_Utils::typeIsCanvasWidget(lvt)) {
-                object->showPropertiesPagedDialog(ClassPropDlg::page_gen);
+                object->showPropertiesPagedDialog(ClassPropertiesDialog::page_gen);
             } else if (umlType == UMLObject::ot_EnumLiteral) {
                 // Show the Enum Literal Dialog
                 UMLEnumLiteral* selectedEnumLiteral = static_cast<UMLEnumLiteral*>(object);
@@ -735,9 +800,9 @@ void UMLListView::slotDiagramCreated(Uml::ID::Type id)
         UMLScene *scene = v->umlScene();
         if (scene) {
             const Uml::DiagramType::Enum dt = scene->type();
-            UMLListViewItem* p = findFolderForDiagram(dt);
+            UMLListViewItem* p = findUMLObject(scene->folder());
             UMLListViewItem* item = new UMLListViewItem(p, scene->name(), Model_Utils::convert_DT_LVT(dt), id);
-            setSelected(item, true);
+            item->setSelected(true);
             UMLApp::app()->docWindow()->showDocumentation(scene, false);
         }
     }
@@ -753,6 +818,7 @@ void UMLListView::slotDiagramCreated(Uml::ID::Type id)
 UMLListViewItem* UMLListView::determineParentItem(UMLObject* object) const
 {
     UMLListViewItem* parentItem = 0;
+    UMLPackage*      pkg = 0;
     UMLListViewItem* current = (UMLListViewItem*) currentItem();
     UMLListViewItem::ListViewType lvt = UMLListViewItem::lvt_Unknown;
     if (current)
@@ -776,8 +842,8 @@ UMLListViewItem* UMLListView::determineParentItem(UMLObject* object) const
     case UMLObject::ot_Stereotype:
         return 0;  // currently no representation in list view
         break;
-    default: {
-        UMLPackage *pkg = object->umlPackage();
+    default:
+        pkg = object->umlPackage();
         if (pkg) {
             UMLListViewItem* pkgItem = findUMLObject(pkg);
             if (pkgItem == 0)
@@ -796,8 +862,7 @@ UMLListViewItem* UMLListView::determineParentItem(UMLObject* object) const
             Uml::ModelType::Enum guess = Model_Utils::guessContainer(object);
             parentItem = m_lv[guess];
         }
-    }
-    break;
+        break;
     }
     return parentItem;
 }
@@ -845,7 +910,16 @@ void UMLListView::slotObjectCreated(UMLObject* object)
         newItem->setIcon(icon);
         return;
     }
-    UMLListViewItem* parentItem = determineParentItem(object);
+    UMLListViewItem* parentItem = 0;
+    UMLPackage *p = object->umlPackage();
+    if (p) {
+        parentItem = findUMLObject(p);
+        if (parentItem == 0)
+            parentItem = determineParentItem(object);
+    } else {
+        uWarning() << object->name() << " : umlPackage not set on object";
+        parentItem = determineParentItem(object);
+    }
     if (parentItem == 0)
         return;
     UMLObject::ObjectType type = object->baseType();
@@ -857,7 +931,7 @@ void UMLListView::slotObjectCreated(UMLObject* object)
         UMLFolder *f = static_cast<UMLFolder*>(object);
         QString folderFile = f->folderFile();
         if (!folderFile.isEmpty())
-            name.append(" (" + folderFile + ')');
+            name.append(QLatin1String(" (") + folderFile + QLatin1Char(')'));
     }
     newItem = new UMLListViewItem(parentItem, name, lvt, object);
     if (mayHaveChildItems(type)) {
@@ -871,7 +945,7 @@ void UMLListView::slotObjectCreated(UMLObject* object)
     scrollToItem(newItem);
     newItem->setOpen(true);
     clearSelection();
-    setSelected(newItem, true);
+    newItem->setSelected(true);
     UMLApp::app()->docWindow()->showDocumentation(object, false);
 }
 
@@ -935,6 +1009,7 @@ void UMLListView::connectNewObjectsSlots(UMLObject* object)
     case UMLObject::ot_Actor:
     case UMLObject::ot_UseCase:
     case UMLObject::ot_Component:
+    case UMLObject::ot_Port:
     case UMLObject::ot_Artifact:
     case UMLObject::ot_Node:
     case UMLObject::ot_Folder:
@@ -1006,7 +1081,7 @@ void UMLListView::childObjectAdded(UMLClassifierListItem* child, UMLClassifier*
         if (! m_doc->loading()) {
             scrollToItem(childItem);
             clearSelection();
-            setSelected(childItem, true);
+            childItem->setSelected(true);
         }
         connectNewObjectsSlots(child);
     }
@@ -1127,6 +1202,7 @@ UMLListViewItem * UMLListView::findUMLObjectInFolder(UMLListViewItem* folder, UM
         case UMLListViewItem::lvt_Package :
         case UMLListViewItem::lvt_Subsystem :
         case UMLListViewItem::lvt_Component :
+        case UMLListViewItem::lvt_Port :
         case UMLListViewItem::lvt_Node :
         case UMLListViewItem::lvt_Artifact :
         case UMLListViewItem::lvt_Interface :
@@ -1323,7 +1399,7 @@ void UMLListView::setView(UMLView * view)
         return;
     UMLListViewItem * temp = findView(view);
     if (temp)
-        setSelected(temp, true);
+        temp->setSelected(true);
 }
 
 /**
@@ -1353,7 +1429,7 @@ void UMLListView::mouseDoubleClickEvent(QMouseEvent * me)
     }
 
     UMLObject::ObjectType type = object->baseType();
-    int page = ClassPropDlg::page_gen;
+    int page = ClassPropertiesDialog::page_gen;
     if (Model_Utils::isClassifierListitem(type)) {
         object = (UMLObject *)object->parent();
     }
@@ -1361,21 +1437,21 @@ void UMLListView::mouseDoubleClickEvent(QMouseEvent * me)
     switch (type) {
 
     case UMLObject::ot_Attribute:
-        page = ClassPropDlg::page_att;
+        page = ClassPropertiesDialog::page_att;
         break;
     case UMLObject::ot_Operation:
-        page = ClassPropDlg::page_op;
+        page = ClassPropertiesDialog::page_op;
         break;
     case UMLObject::ot_EntityAttribute:
-        page = ClassPropDlg::page_entatt;
+        page = ClassPropertiesDialog::page_entatt;
         break;
     case UMLObject::ot_UniqueConstraint:
     case UMLObject::ot_ForeignKeyConstraint:
     case UMLObject::ot_CheckConstraint:
-        page = ClassPropDlg::page_constraint;
+        page = ClassPropertiesDialog::page_constraint;
         break;
     default:
-        page = ClassPropDlg::page_gen;
+        page = ClassPropertiesDialog::page_gen;
         break;
     }
 
@@ -1433,8 +1509,9 @@ bool UMLListView::acceptDrag(QDropEvent* event) const
                 srcType = data->type;
                 accept = Model_Utils::typeIsAllowedInType(srcType, dstType);
 
-                // disallow drop of any child element is not allowed
-                if (!accept) break;
+                // disallow drop if any child element is not allowed
+                if (!accept)
+                    break;
             }
             break;
         }
@@ -1572,6 +1649,16 @@ UMLListViewItem * UMLListView::moveObject(Uml::ID::Type srcId, UMLListViewItem::
             addAtContainer(newItem, newParent);
         }
         break;
+    case UMLListViewItem::lvt_Port:
+        if (newParentType == UMLListViewItem::lvt_Component) {
+            newItem = move->deepCopy(newParent);
+            if (m_doc->loading())         // deletion is not safe while loading
+                move->setVisible(false);  // (the <listview> XMI may be corrupted)
+            else
+                delete move;
+            addAtContainer(newItem, newParent);
+        }
+        break;
     case UMLListViewItem::lvt_Deployment_Folder:
     case UMLListViewItem::lvt_Node:
     case UMLListViewItem::lvt_Deployment_Diagram:
@@ -1866,6 +1953,7 @@ UMLListViewItem* UMLListView::determineParentItem(UMLListViewItem::ListViewType
         break;
     case UMLListViewItem::lvt_Component_Diagram:
     case UMLListViewItem::lvt_Component:
+    case UMLListViewItem::lvt_Port:
     case UMLListViewItem::lvt_Artifact:
         parent = m_lv[Uml::ModelType::Component];
         break;
@@ -1919,19 +2007,19 @@ void UMLListView::focusOutEvent(QFocusEvent * fe)
     QTreeWidget::focusOutEvent(fe);
 }
 
-void UMLListView::contextMenuEvent(QContextMenuEvent *me)
+void UMLListView::contextMenuEvent(QContextMenuEvent *event)
 {
     // Get the UMLListViewItem at the point where the mouse pointer was pressed
-    UMLListViewItem * item = static_cast<UMLListViewItem*>(itemAt(me->pos()));
+    UMLListViewItem * item = static_cast<UMLListViewItem*>(itemAt(event->pos()));
     if (item) {
         const UMLListViewItem::ListViewType type = item->type();
         ListPopupMenu popup(this, type, item->umlObject());
-        QAction *triggered = popup.exec(me->globalPos());
-        slotMenuSelection(triggered);
-        me->accept();
+        QAction *triggered = popup.exec(event->globalPos());
+        slotMenuSelection(triggered, event->globalPos());
+        event->accept();
     }
 
-    QTreeWidget::contextMenuEvent(me);
+    QTreeWidget::contextMenuEvent(event);
 }
 
 /**
@@ -2082,9 +2170,9 @@ void UMLListView::addNewItem(UMLListViewItem *parentItem, UMLListViewItem::ListV
         }
 
         if (type == UMLListViewItem::lvt_Subsystem) {
-            object->setStereotypeCmd("subsystem");
+            object->setStereotypeCmd(QLatin1String("subsystem"));
         } else if (Model_Utils::typeIsFolder(type)) {
-            object->setStereotypeCmd("folder");
+            object->setStereotypeCmd(QLatin1String("folder"));
         }
     }
 }
@@ -2148,6 +2236,7 @@ bool UMLListView::isUnique(UMLListViewItem * item, const QString &name)
     case UMLListViewItem::lvt_Enum:
     case UMLListViewItem::lvt_Entity:
     case UMLListViewItem::lvt_Component:
+    case UMLListViewItem::lvt_Port:
     case UMLListViewItem::lvt_Subsystem:
     case UMLListViewItem::lvt_Logical_Folder:
     case UMLListViewItem::lvt_UseCase_Folder:
@@ -2192,7 +2281,7 @@ bool UMLListView::isUnique(UMLListViewItem * item, const QString &name)
  */
 void UMLListView::saveToXMI(QDomDocument & qDoc, QDomElement & qElement)
 {
-    QDomElement listElement = qDoc.createElement("listview");
+    QDomElement listElement = qDoc.createElement(QLatin1String("listview"));
     m_rv->saveToXMI(qDoc, listElement);
     qElement.appendChild(listElement);
 }
@@ -2206,9 +2295,9 @@ bool UMLListView::loadFromXMI(QDomElement & element)
     QDomElement domElement = node.toElement();
     m_doc->writeToStatusBar(i18n("Loading listview..."));
     while (!domElement.isNull()) {
-        if (domElement.tagName() == "listitem") {
-            QString type = domElement.attribute("type", "-1");
-            if (type == "-1")
+        if (domElement.tagName() == QLatin1String("listitem")) {
+            QString type = domElement.attribute(QLatin1String("type"), QLatin1String("-1"));
+            if (type == QLatin1String("-1"))
                 return false;
             UMLListViewItem::ListViewType lvType = (UMLListViewItem::ListViewType)type.toInt();
             if (lvType == UMLListViewItem::lvt_View) {
@@ -2233,15 +2322,15 @@ bool UMLListView::loadChildrenFromXMI(UMLListViewItem * parent, QDomElement & el
     QDomElement domElement = node.toElement();
     while (!domElement.isNull()) {
         node = domElement.nextSibling();
-        if (domElement.tagName() != "listitem") {
+        if (domElement.tagName() != QLatin1String("listitem")) {
             domElement = node.toElement();
             continue;
         }
-        QString id = domElement.attribute("id", "-1");
-        QString type = domElement.attribute("type", "-1");
-        QString label = domElement.attribute("label", "");
-        QString open = domElement.attribute("open", "1");
-        if (type == "-1")
+        QString id = domElement.attribute(QLatin1String("id"), QLatin1String("-1"));
+        QString type = domElement.attribute(QLatin1String("type"), QLatin1String("-1"));
+        QString label = domElement.attribute(QLatin1String("label"));
+        QString open = domElement.attribute(QLatin1String("open"), QLatin1String("1"));
+        if (type == QLatin1String("-1"))
             return false;
         UMLListViewItem::ListViewType lvType = (UMLListViewItem::ListViewType)type.toInt();
         bool bOpen = (bool)open.toInt();
@@ -2312,6 +2401,7 @@ bool UMLListView::loadChildrenFromXMI(UMLListViewItem * parent, QDomElement & el
         case UMLListViewItem::lvt_Package:
         case UMLListViewItem::lvt_Subsystem:
         case UMLListViewItem::lvt_Component:
+        case UMLListViewItem::lvt_Port:
         case UMLListViewItem::lvt_Node:
         case UMLListViewItem::lvt_Artifact:
         case UMLListViewItem::lvt_Logical_Folder:
@@ -2347,7 +2437,7 @@ bool UMLListView::loadChildrenFromXMI(UMLListViewItem * parent, QDomElement & el
                     item = newItem;
                     if (item) {
                         DEBUG(DBG_SRC) << "Attempted reparenting of " << item->text(0)
-                                       << "(current parent: " << (itmParent ? itmParent->text(0) : "0")
+                                       << "(current parent: " << (itmParent ? itmParent->text(0) : QLatin1String("0"))
                                        << ", new parent: " << parent->text(0) << ")";
                     }
                 }
