@@ -89,7 +89,6 @@ public:
 
     bool isUnique(UMLListViewItem * item, const QString &name);
 
-    void setSelected(UMLListViewItem * item, bool state) { setItemSelected((QTreeWidgetItem*)item, state);  }
     void setStartedCut(bool startedCut);
     void setStartedCopy(bool startedCopy);
     bool startedCopy() const;
@@ -121,7 +120,7 @@ protected:
     void mouseMoveEvent(QMouseEvent *me);
     void mouseDoubleClickEvent(QMouseEvent *me);
     void focusOutEvent (QFocusEvent *fe);
-    void contextMenuEvent(QContextMenuEvent *me);
+    void contextMenuEvent(QContextMenuEvent *event);
     UMLDragData* getDragData();
 
     bool acceptDrag(QDropEvent* event) const;
@@ -157,7 +156,7 @@ public slots:
 
     void slotObjectChanged();
 
-    void slotMenuSelection(QAction* action);
+    void slotMenuSelection(QAction* action, const QPoint &position = QPoint());
 
     void slotDropped(QDropEvent* de, UMLListViewItem* target);
 
@@ -187,6 +186,8 @@ private:
     bool m_bCreatingChildObject;  ///< when creating an attribute or an operation to stop it adding a second listViewItem
     QPoint m_dragStartPosition;
     QMimeData* m_dragCopyData;
+    QList<QTreeWidgetItem*> m_findItems;
+    int n_findIndex;
 };
 
 #endif
