@@ -15,7 +15,6 @@
 #include "associationwidgetlist.h"
 #include "basictypes.h"
 #include "classifierwidget.h"
-#include "floatingtextwidget.h"
 #include "messagewidgetlist.h"
 #include "optionstate.h"
 #include "umlobject.h"
@@ -61,6 +60,7 @@ class QHideEvent;
 class QMouseEvent;
 class QPrinter;
 class QShowEvent;
+class UMLScenePrivate;
 
 // migration wrapper for QGraphicsScene items
 typedef QList<QGraphicsItem*> UMLSceneItemList;
@@ -154,7 +154,6 @@ public:
     AssociationWidgetList& associationList();
     UMLWidgetList& widgetList();
     MessageWidgetList& messageList();
-    UMLObjectList umlObjects();
 
     bool isOpen() const;
     void setIsOpen(bool isOpen);
@@ -170,7 +169,6 @@ public:
     void checkMessages(ObjectWidget * w);
 
     UMLWidget* findWidget(Uml::ID::Type id);
-    UMLWidget* findWidgetByLocalId(Uml::ID::Type id);
 
     AssociationWidget* findAssocWidget(Uml::ID::Type id);
     AssociationWidget* findAssocWidget(Uml::AssociationType::Enum at,
@@ -368,6 +366,7 @@ protected:
     bool m_bStartedCut;  ///< Flag if view/children started cut operation.
 
 private:
+    UMLScenePrivate *m_d;
     static const qreal defaultCanvasSize;  ///< The default size of a diagram in pixels.
     static bool m_showDocumentationIndicator; ///< Status of documentation indicator
 
@@ -418,7 +417,6 @@ signals:
     void sigLineColorChanged(Uml::ID::Type);
     void sigTextColorChanged(Uml::ID::Type);
     void sigLineWidthChanged(Uml::ID::Type);
-    void sigClearAllSelected();
     void sigSnapToGridToggled(bool);
     void sigSnapComponentSizeToGridToggled(bool);
     void sigShowGridToggled(bool);
