@@ -12,7 +12,9 @@
 #include "classifierwidget.h"
 
 // app includes
+#include "floatingtextwidget.h"
 #include "associationwidget.h"
+#include "associationline.h"
 #include "classifier.h"
 #include "cmds.h"
 #include "debug_utils.h"
@@ -31,16 +33,17 @@ DEBUG_REGISTER_DISABLED(ClassifierWidget)
 
 const int ClassifierWidget::MARGIN = 5;
 const int ClassifierWidget::CIRCLE_SIZE = 30;
+const int ClassifierWidget::SOCKET_INCREMENT = 10;
 
 /**
  * Constructs a ClassifierWidget.
  *
  * @param scene   The parent of this ClassifierWidget.
- * @param o       The UMLObject to represent.
+ * @param c       The UMLClassifier to represent.
  */
 ClassifierWidget::ClassifierWidget(UMLScene * scene, UMLClassifier *c)
   : UMLWidget(scene, WidgetBase::wt_Class, c),
-    m_pAssocWidget(0)
+    m_pAssocWidget(0), m_pInterfaceName(0)
 {
     const Settings::OptionState& ops = m_scene->optionState();
     setVisualPropertyCmd(ShowVisibility, ops.classState.showVisibility);
@@ -84,6 +87,10 @@ ClassifierWidget::~ClassifierWidget()
 {
     if (m_pAssocWidget)
         m_pAssocWidget->removeAssocClassLine();
+    if (m_pInterfaceName) {
+        delete m_pInterfaceName;
+        m_pInterfaceName = 0;
+    }
 }
 
 /**
@@ -121,8 +128,8 @@ void ClassifierWidget::setVisualProperties(VisualProperties properties)
 /**
  * @return The status of the property passed in.
  *
- * @note Use @ref attributeSignatureType() and @ref
- *       operationSignatureType() to get signature status.  This
+ * @note Use @ref attributeSignature() and @ref
+ *       operationSignature() to get signature status.  This
  *       method only indicates whether signature is visible or not.
  */
 bool ClassifierWidget::visualProperty(VisualProperty property) const
@@ -176,8 +183,7 @@ void ClassifierWidget::setVisualPropertyCmd(VisualProperty property, bool enable
         if (!enable) {
             m_attributeSignature = visualProperty(ShowVisibility) ?
                 Uml::SignatureType::NoSig : Uml::SignatureType::NoSigNoVis;
-        }
-        else {
+        } else {
             m_attributeSignature = visualProperty(ShowVisibility) ?
                 Uml::SignatureType::ShowSig : Uml::SignatureType::SigNoVis;
         }
@@ -189,8 +195,7 @@ void ClassifierWidget::setVisualPropertyCmd(VisualProperty property, bool enable
         if (!enable) {
             m_operationSignature = visualProperty(ShowVisibility) ?
                 Uml::SignatureType::NoSig : Uml::SignatureType::NoSigNoVis;
-        }
-        else {
+        } else {
             m_operationSignature = visualProperty(ShowVisibility) ?
                 Uml::SignatureType::ShowSig : Uml::SignatureType::SigNoVis;
         }
@@ -202,13 +207,24 @@ void ClassifierWidget::setVisualPropertyCmd(VisualProperty property, bool enable
         // Now just update flag and use base method for actual work.
         if (enable) {
             m_visualProperties |= property;
-        }
-        else {
+        } else {
             m_visualProperties &= ~property;
         }
         setShowStereotype(enable);
     }
 
+    else if (property == DrawAsCircle) {
+        // Don't do anything if the flag status is same.
+        if (visualProperty(property) == enable)
+            return;
+        if (enable) {
+            m_visualProperties |= property;
+        } else {
+            m_visualProperties &= ~property;
+        }
+        setDrawAsCircle(enable);
+    }
+
     // Some other flag.
     else {
         // Don't do anything if the flag status is same.
@@ -219,8 +235,7 @@ void ClassifierWidget::setVisualPropertyCmd(VisualProperty property, bool enable
         // Call setVisualProperties appropriately based on enbable.
         if (enable) {
             setVisualProperties(visualProperties() | property);
-        }
-        else {
+        } else {
             setVisualProperties(visualProperties() & ~property);
         }
     }
@@ -340,7 +355,7 @@ void ClassifierWidget::setOperationSignature(Uml::SignatureType::Enum sig)
  * Sets whether to show attribute signature
  * Only applies when m_umlObject->getBaseType() is ot_Class.
  *
- * @param _show   True if attribute signatures shall be shown.
+ * @param _status  True if attribute signatures shall be shown.
  */
 void ClassifierWidget::setShowAttSigs(bool _status)
 {
@@ -386,10 +401,13 @@ void ClassifierWidget::toggleShowAttSigs()
  * Return the number of displayed members of the given ObjectType.
  * Takes into consideration m_showPublicOnly but not other settings.
  */
-int ClassifierWidget::displayedMembers(UMLObject::ObjectType ot)
+int ClassifierWidget::displayedMembers(UMLObject::ObjectType ot) const
 {
     int count = 0;
-    UMLClassifierListItemList list = classifier()->getFilteredList(ot);
+    UMLClassifier *classifier = this->classifier();
+    if (!classifier)
+        return count;
+    UMLClassifierListItemList list = classifier->getFilteredList(ot);
     foreach (UMLClassifierListItem *m, list) {
       if (!(visualProperty(ShowPublicOnly) && m->visibility() != Uml::Visibility::Public))
             count++;
@@ -400,7 +418,16 @@ int ClassifierWidget::displayedMembers(UMLObject::ObjectType ot)
 /**
  * Overrides method from UMLWidget.
  */
-QSizeF ClassifierWidget::minimumSize()
+QSizeF ClassifierWidget::minimumSize() const
+{
+    return calculateSize();
+}
+
+/**
+ * Calculate content related size of widget.
+ * Overrides method from UMLWidget.
+ */
+QSizeF ClassifierWidget::calculateSize(bool withExtensions /* = true */) const
 {
     if (!m_umlObject) {
         return UMLWidget::minimumSize();
@@ -474,16 +501,17 @@ QSizeF ClassifierWidget::minimumSize()
         }
     }
 
-    // consider template box _as last_ !
-    QSize templatesBoxSize = calculateTemplatesBoxSize();
-    if (templatesBoxSize.width() != 0) {
-        // add width to largest 'word'
-        width += templatesBoxSize.width() / 2;
+    if (withExtensions) {
+        // consider template box _as last_ !
+        QSize templatesBoxSize = calculateTemplatesBoxSize();
+        if (templatesBoxSize.width() != 0) {
+            // add width to largest 'word'
+            width += templatesBoxSize.width() / 2;
+        }
+        if (templatesBoxSize.height() != 0) {
+            height += templatesBoxSize.height() - MARGIN;
+        }
     }
-    if (templatesBoxSize.height() != 0) {
-        height += templatesBoxSize.height() - MARGIN;
-    }
-
 
     // allow for height margin
     if (!visualProperty(ShowOperations) && !visualProperty(ShowAttributes) && !m_showStereotype) {
@@ -502,7 +530,7 @@ QSizeF ClassifierWidget::minimumSize()
  *
  * @return  QSize of the templates flap.
  */
-QSize ClassifierWidget::calculateTemplatesBoxSize()
+QSize ClassifierWidget::calculateTemplatesBoxSize() const
 {
     UMLTemplateList list = classifier()->getTemplateList();
     int count = list.count();
@@ -532,7 +560,7 @@ QSize ClassifierWidget::calculateTemplatesBoxSize()
 /**
  * Return the number of displayed attributes.
  */
-int ClassifierWidget::displayedAttributes()
+int ClassifierWidget::displayedAttributes() const
 {
     if (!visualProperty(ShowAttributes))
         return 0;
@@ -542,7 +570,7 @@ int ClassifierWidget::displayedAttributes()
 /**
  * Return the number of displayed operations.
  */
-int ClassifierWidget::displayedOperations()
+int ClassifierWidget::displayedOperations() const
 {
     if (!visualProperty(ShowOperations))
         return 0;
@@ -590,18 +618,18 @@ void ClassifierWidget::paint(QPainter *painter, const QStyleOptionGraphicsItem *
         return;
     }
 
-    // Draw the bounding rectangle
+    // Draw the main bounding rectangle (without template box)
     QSize templatesBoxSize = calculateTemplatesBoxSize();
-    int m_bodyOffsetY = 0;
+    int bodyOffsetY = 0;
     if (templatesBoxSize.height() > 0)
-        m_bodyOffsetY += templatesBoxSize.height() - MARGIN;
+        bodyOffsetY += templatesBoxSize.height() - MARGIN;
     int w = width();
     if (templatesBoxSize.width() > 0)
         w -= templatesBoxSize.width() / 2;
     int h = height();
     if (templatesBoxSize.height() > 0)
         h -= templatesBoxSize.height() - MARGIN;
-    painter->drawRect(0, m_bodyOffsetY, w, h);
+    painter->drawRect(0, bodyOffsetY, w, h);
 
     QFont font = UMLWidget::font();
     font.setUnderline(false);
@@ -649,8 +677,8 @@ void ClassifierWidget::paint(QPainter *painter, const QStyleOptionGraphicsItem *
     } else if (showStereotype) {
         painter->setFont(font);
         stereo = m_umlObject->stereotype(true);
-        painter->drawText(textX, m_bodyOffsetY, textWidth, fontHeight, Qt::AlignCenter, stereo);
-        m_bodyOffsetY += fontHeight;
+        painter->drawText(textX, bodyOffsetY, textWidth, fontHeight, Qt::AlignCenter, stereo);
+        bodyOffsetY += fontHeight;
     }
 
     // draw name
@@ -662,11 +690,11 @@ void ClassifierWidget::paint(QPainter *painter, const QStyleOptionGraphicsItem *
     }
     font.setItalic(m_umlObject->isAbstract());
     painter->setFont(font);
-    painter->drawText(textX, m_bodyOffsetY, textWidth, nameHeight, Qt::AlignCenter, name);
+    painter->drawText(textX, bodyOffsetY, textWidth, nameHeight, Qt::AlignCenter, name);
     if (!showNameOnly) {
-        m_bodyOffsetY += fontHeight;
+        bodyOffsetY += fontHeight;
         setPenFromSettings(painter);
-        painter->drawLine(0, m_bodyOffsetY, w, m_bodyOffsetY);
+        painter->drawLine(0, bodyOffsetY, w, bodyOffsetY);
         painter->setPen(textColor());
     }
     font.setBold(false);
@@ -677,24 +705,24 @@ void ClassifierWidget::paint(QPainter *painter, const QStyleOptionGraphicsItem *
     const int numAtts = displayedAttributes();
     if (visualProperty(ShowAttributes)) {
         drawMembers(painter, UMLObject::ot_Attribute, m_attributeSignature, textX,
-                    m_bodyOffsetY, fontHeight);
+                    bodyOffsetY, fontHeight);
     }
 
     // draw dividing line between attributes and operations
     if (!showNameOnly) {
         if (numAtts == 0)
-            m_bodyOffsetY += fontHeight / 2;  // no atts, so just add a bit of space
+            bodyOffsetY += fontHeight / 2;  // no atts, so just add a bit of space
         else
-            m_bodyOffsetY += fontHeight * numAtts;
+            bodyOffsetY += fontHeight * numAtts;
         setPenFromSettings(painter);
-        painter->drawLine(0, m_bodyOffsetY, w, m_bodyOffsetY);
+        painter->drawLine(0, bodyOffsetY, w, bodyOffsetY);
         painter->setPen(QPen(textColor()));
     }
 
     // draw operations
     if (visualProperty(ShowOperations)) {
         drawMembers(painter, UMLObject::ot_Operation, m_operationSignature, textX,
-                    m_bodyOffsetY, fontHeight);
+                    bodyOffsetY, fontHeight);
     }
 
     UMLWidget::paint(painter, option, widget);
@@ -705,39 +733,89 @@ void ClassifierWidget::paint(QPainter *painter, const QStyleOptionGraphicsItem *
  */
 QPainterPath ClassifierWidget::shape() const
 {
+    QPainterPath path;
     if (classifier()->isInterface() && visualProperty(DrawAsCircle)) {
-        QPainterPath path;
-        path.addEllipse(width()/2 - CIRCLE_SIZE/2, 0, CIRCLE_SIZE, CIRCLE_SIZE);
-        path.addRect(0, CIRCLE_SIZE, width(), height() - CIRCLE_SIZE);
+        path.addEllipse(rect());
         return path;
     }
-    return UMLWidget::shape();
+    QSizeF mainSize = calculateSize(false);
+    QSize templatesBoxSize = calculateTemplatesBoxSize();
+    qreal mainY = 0.0;
+    if (templatesBoxSize.height() > 0) {
+        mainY += templatesBoxSize.height() - MARGIN;
+        path.addRect(QRectF(mainSize.width() - templatesBoxSize.width() / 2, 0.0,
+                            templatesBoxSize.width(), templatesBoxSize.height()));
+    }
+    path.addRect(QRectF(0.0, mainY, mainSize.width(), mainSize.height()));
+    return path;
 }
 
 /**
- * Draws the interface as a circle with name underneath.
+ * Draws the interface as a circle.
  * Only applies when m_umlObject->getBaseType() is ot_Interface.
  */
 void ClassifierWidget::drawAsCircle(QPainter *painter, const QStyleOptionGraphicsItem *option)
 {
-    int w = width();
+    const int w = width();
 
-    const QFontMetrics &fm = getFontMetrics(FT_NORMAL);
-    const int fontHeight  = fm.lineSpacing();
-    QString name;
-    if (visualProperty(ShowPackage)) {
-        name = m_umlObject->fullyQualifiedName();
-    } else {
-        name = this->name();
+    painter->drawEllipse(w/2 - CIRCLE_SIZE/2, SOCKET_INCREMENT / 2, CIRCLE_SIZE, CIRCLE_SIZE);
+    if (m_Assocs.size() > 1) {
+        // Draw socket for required interface.
+        const qreal angleSpan = 180;   // 360.0 / (m_Assocs.size() + 1.0);
+        const int arcDiameter = CIRCLE_SIZE + SOCKET_INCREMENT;
+        QRect requireArc(w/2 - arcDiameter/2, 0, arcDiameter, arcDiameter);
+        const QPointF center(x() + w/2, y() + arcDiameter/2);
+        const qreal cX = center.x();
+        const qreal cY = center.y();
+        foreach (AssociationWidget *aw, m_Assocs) {
+            const Uml::AssociationType::Enum aType = aw->associationType();
+            if (aType == Uml::AssociationType::UniAssociation ||
+                   aType == Uml::AssociationType::Association)  // provider
+                continue;
+            UMLWidget *otherEnd = aw->widgetForRole(Uml::RoleType::A);
+            const WidgetBase::WidgetType oType = otherEnd->baseType();
+            if (oType != WidgetBase::wt_Component && oType != WidgetBase::wt_Port)
+                continue;
+
+            AssociationLine *assocLine = aw->associationLine();
+            const QPointF p(assocLine->endPoint());
+            const qreal tolerance = 18.0;
+            bool drawArc = true;
+            qreal midAngle;
+            if (p.x() < cX - tolerance) {
+                if (p.y() < cY - tolerance)
+                    midAngle = 135;
+                else if (p.y() > cY + tolerance)
+                    midAngle = 225;
+                else
+                    midAngle = 180;
+            } else if (p.x() > cX + tolerance) {
+                if (p.y() < cY - tolerance)
+                    midAngle = 45;
+                else if (p.y() > cY + tolerance)
+                    midAngle = 315;
+                else
+                    midAngle = 0;
+            } else {
+                if (p.y() < cY - tolerance)
+                    midAngle = 90;
+                else if (p.y() > cY + tolerance)
+                    midAngle = 270;
+                else
+                    drawArc = false;
+            }
+            if (drawArc) {
+                // uDebug() << "number of assocs: " << m_Assocs.size()
+                //          << ", p: " << p << ", center: " << center
+                //          << ", midAngle: " << midAngle << ", angleSpan: " << angleSpan;
+                painter->drawArc(requireArc, 16 * (midAngle - angleSpan/2), 16 * angleSpan);
+            } else {
+                uError() << "socket: assocLine endPoint " << p
+                         << " too close to own center" << center;
+            }
+        }
     }
 
-    painter->drawEllipse(w/2 - CIRCLE_SIZE/2, 0, CIRCLE_SIZE, CIRCLE_SIZE);
-    painter->setPen(QPen(textColor()));
-
-    QFont font = UMLWidget::font();
-    painter->setFont(font);
-    painter->drawText(0, CIRCLE_SIZE, w, fontHeight, Qt::AlignCenter, name);
-
     UMLWidget::paint(painter, option);
 }
 
@@ -745,26 +823,12 @@ void ClassifierWidget::drawAsCircle(QPainter *painter, const QStyleOptionGraphic
  * Calculates the size of the object when drawn as a circle.
  * Only applies when m_umlObject->getBaseType() is ot_Interface.
  */
-QSize ClassifierWidget::calculateAsCircleSize()
+QSize ClassifierWidget::calculateAsCircleSize() const
 {
-    const QFontMetrics &fm = UMLWidget::getFontMetrics(UMLWidget::FT_ITALIC_UNDERLINE);
-    const int fontHeight = fm.lineSpacing();
-
-    int height = CIRCLE_SIZE + fontHeight;
-
-    int width = CIRCLE_SIZE;
-    QString displayedName;
-    if (visualProperty(ShowPackage)) {
-        displayedName = m_umlObject->fullyQualifiedName();
-    } else {
-        displayedName = m_umlObject->name();
-    }
-    const int nameWidth = fm.size(0, displayedName).width();
-    if (nameWidth > width)
-        width = nameWidth;
-    width += MARGIN * 2;
-
-    return QSize(width, height);
+    int circleSize = CIRCLE_SIZE;
+    if (m_Assocs.size() > 1)
+        circleSize += SOCKET_INCREMENT;
+    return QSize(circleSize, circleSize);
 }
 
 /**
@@ -803,6 +867,38 @@ void ClassifierWidget::drawMembers(QPainter * painter, UMLObject::ObjectType ot,
     painter->setClipping(false);
 }
 
+/**
+ * Override method from UMLWidget in order to additionally check m_pInterfaceName.
+ *
+ * @param p Point to be checked.
+ *
+ * @return 'this' if UMLWidget::onWidget(p) returns non NULL;
+ *         m_pInterfaceName if m_pName is non NULL and
+ *         m_pInterfaceName->onWidget(p) returns non NULL; else NULL.
+ */
+UMLWidget* ClassifierWidget::onWidget(const QPointF &p)
+{
+    if (UMLWidget::onWidget(p) != NULL)
+        return this;
+    if (getDrawAsCircle() && m_pInterfaceName) {
+        uDebug() << "floatingtext: " << m_pInterfaceName->text();
+        return m_pInterfaceName->onWidget(p);
+    }
+    return NULL;
+}
+
+/**
+ * Reimplement function from UMLWidget.
+ */
+UMLWidget* ClassifierWidget::widgetWithID(Uml::ID::Type id)
+{
+    if (UMLWidget::widgetWithID(id))
+        return this;
+    if (getDrawAsCircle() && m_pInterfaceName && m_pInterfaceName->widgetWithID(id))
+        return m_pInterfaceName;
+    return NULL;
+}
+
 /**
  * Sets whether to draw as circle.
  * Only applies when m_umlObject->getBaseType() is ot_Interface.
@@ -812,6 +908,29 @@ void ClassifierWidget::drawMembers(QPainter * painter, UMLObject::ObjectType ot,
 void ClassifierWidget::setDrawAsCircle(bool drawAsCircle)
 {
     setVisualPropertyCmd(DrawAsCircle, drawAsCircle);
+    const int circleSize = CIRCLE_SIZE + SOCKET_INCREMENT;
+    if (drawAsCircle) {
+        setX(x() + (width()/2 - circleSize/2));
+        setY(y() + (height()/2 - circleSize/2));
+        setSize(circleSize, circleSize);
+        if (m_pInterfaceName) {
+            m_pInterfaceName->show();
+        } else {
+            m_pInterfaceName = new FloatingTextWidget(m_scene, Uml::TextRole::Floating, name());
+            m_pInterfaceName->setParentItem(this);
+            m_pInterfaceName->setText(name());  // to get geometry update
+            m_pInterfaceName->setX(circleSize/2 - m_pInterfaceName->width() / 2);
+            m_pInterfaceName->setY(circleSize + SOCKET_INCREMENT);
+        }
+        m_resizable = false;
+    } else {
+        setSize(ClassifierWidget::minimumSize());
+        setX(x() - (width()/2 - circleSize/2));
+        setY(y() - (height()/2 - circleSize/2));
+        if (m_pInterfaceName)
+            m_pInterfaceName->hide();
+        m_resizable = true;
+    }
     updateGeometry();
     update();
 }
@@ -848,7 +967,7 @@ void ClassifierWidget::changeToClass()
 {
     m_baseType = WidgetBase::wt_Class;
     classifier()->setBaseType(UMLObject::ot_Class);
-
+    setVisualPropertyCmd(DrawAsCircle, false);
     const Settings::OptionState& ops = m_scene->optionState();
     setVisualProperty(ShowAttributes, ops.classState.showAtts);
     setVisualProperty(ShowStereotype, ops.classState.showStereoType);
@@ -882,8 +1001,8 @@ void ClassifierWidget::changeToInterface()
  * updated.
  * TODO: This is never called.
  *
- * @param x The x-coordinate.
- * @param y The y-coordinate.
+ *  param x The x-coordinate.
+ *  param y The y-coordinate.
  */
 //void ClassifierWidget::adjustAssociations(int x, int y)
 //{
@@ -908,14 +1027,14 @@ bool ClassifierWidget::loadFromXMI(QDomElement & qElement)
         return false;
     }
 
-    QString showatts = qElement.attribute("showattributes", "0");
-    QString showops = qElement.attribute("showoperations", "1");
-    QString showpubliconly = qElement.attribute("showpubliconly", "0");
-    QString showattsigs = qElement.attribute("showattsigs", "600");
-    QString showopsigs = qElement.attribute("showopsigs", "600");
-    QString showpackage = qElement.attribute("showpackage", "0");
-    QString showscope = qElement.attribute("showscope", "0");
-    QString drawascircle = qElement.attribute("drawascircle", "0");
+    QString showatts = qElement.attribute(QLatin1String("showattributes"), QLatin1String("0"));
+    QString showops = qElement.attribute(QLatin1String("showoperations"), QLatin1String("1"));
+    QString showpubliconly = qElement.attribute(QLatin1String("showpubliconly"), QLatin1String("0"));
+    QString showattsigs = qElement.attribute(QLatin1String("showattsigs"), QLatin1String("600"));
+    QString showopsigs = qElement.attribute(QLatin1String("showopsigs"), QLatin1String("600"));
+    QString showpackage = qElement.attribute(QLatin1String("showpackage"), QLatin1String("0"));
+    QString showscope = qElement.attribute(QLatin1String("showscope"), QLatin1String("0"));
+    QString drawascircle = qElement.attribute(QLatin1String("drawascircle"), QLatin1String("0"));
 
     setVisualPropertyCmd(ShowAttributes, (bool)showatts.toInt());
     setVisualPropertyCmd(ShowOperations, (bool)showops.toInt());
@@ -927,6 +1046,34 @@ bool ClassifierWidget::loadFromXMI(QDomElement & qElement)
     m_attributeSignature = Uml::SignatureType::fromInt(showattsigs.toInt());
     m_operationSignature = Uml::SignatureType::fromInt(showopsigs.toInt());
 
+    if (!getDrawAsCircle())
+        return true;
+
+    // Optional child element: floatingtext
+    QDomNode node = qElement.firstChild();
+    QDomElement element = node.toElement();
+    if (!element.isNull()) {
+        QString tag = element.tagName();
+        if (tag == QLatin1String("floatingtext")) {
+            if (m_pInterfaceName == NULL) {
+                m_pInterfaceName = new FloatingTextWidget(m_scene,
+                                                          Uml::TextRole::Floating,
+                                                          name(), Uml::ID::Reserved);
+                m_pInterfaceName->setParentItem(this);
+            }
+            if (!m_pInterfaceName->loadFromXMI(element)) {
+                // Most likely cause: The FloatingTextWidget is empty.
+                delete m_pInterfaceName;
+                m_pInterfaceName = NULL;
+            } else {
+                m_pInterfaceName->activate();
+                m_pInterfaceName->update();
+            }
+        } else {
+            uError() << "unknown tag " << tag;
+        }
+    }
+
     return true;
 }
 
@@ -938,21 +1085,25 @@ void ClassifierWidget::saveToXMI(QDomDocument & qDoc, QDomElement & qElement)
     QDomElement conceptElement;
     UMLClassifier *umlc = classifier();
     if (umlc->isInterface())
-        conceptElement = qDoc.createElement("interfacewidget");
+        conceptElement = qDoc.createElement(QLatin1String("interfacewidget"));
     else
-        conceptElement = qDoc.createElement("classwidget");
+        conceptElement = qDoc.createElement(QLatin1String("classwidget"));
     UMLWidget::saveToXMI(qDoc, conceptElement);
-    conceptElement.setAttribute("showoperations", visualProperty(ShowOperations));
-    conceptElement.setAttribute("showpubliconly", visualProperty(ShowPublicOnly));
-    conceptElement.setAttribute("showopsigs",     m_operationSignature);
-    conceptElement.setAttribute("showpackage",    visualProperty(ShowPackage));
-    conceptElement.setAttribute("showscope",      visualProperty(ShowVisibility));
+    conceptElement.setAttribute(QLatin1String("showoperations"), visualProperty(ShowOperations));
+    conceptElement.setAttribute(QLatin1String("showpubliconly"), visualProperty(ShowPublicOnly));
+    conceptElement.setAttribute(QLatin1String("showopsigs"),     m_operationSignature);
+    conceptElement.setAttribute(QLatin1String("showpackage"),    visualProperty(ShowPackage));
+    conceptElement.setAttribute(QLatin1String("showscope"),      visualProperty(ShowVisibility));
     if (! umlc->isInterface()) {
-        conceptElement.setAttribute("showattributes", visualProperty(ShowAttributes));
-        conceptElement.setAttribute("showattsigs",    m_attributeSignature);
+        conceptElement.setAttribute(QLatin1String("showattributes"), visualProperty(ShowAttributes));
+        conceptElement.setAttribute(QLatin1String("showattsigs"),    m_attributeSignature);
+    }
+    if (umlc->isInterface() || umlc->isAbstract()) {
+        conceptElement.setAttribute(QLatin1String("drawascircle"), visualProperty(DrawAsCircle));
+        if (visualProperty(DrawAsCircle) && m_pInterfaceName) {
+            m_pInterfaceName->saveToXMI(qDoc, conceptElement);
+        }
     }
-    if (umlc->isInterface() || umlc->isAbstract())
-        conceptElement.setAttribute("drawascircle", visualProperty(DrawAsCircle));
     qElement.appendChild(conceptElement);
 }
 
