@@ -16,6 +16,7 @@
 #include "umlwidget.h"
 
 class AssociationWidget;
+class FloatingTextWidget;
 class QPainter;
 class UMLClassifier;
 
@@ -65,8 +66,8 @@ public:
     void setVisualPropertyCmd(VisualProperty property, bool enable = true);
     void toggleVisualProperty(VisualProperty property);
 
-    int displayedAttributes();
-    int displayedOperations();
+    int displayedAttributes() const;
+    int displayedOperations() const;
 
     Uml::SignatureType::Enum attributeSignature() const;
     void setAttributeSignature(Uml::SignatureType::Enum sig);
@@ -87,13 +88,18 @@ public:
     AssociationWidget *classAssociationWidget() const;
     void setClassAssociationWidget(AssociationWidget *assocwidget);
 //    virtual void adjustAssociations(int x, int y);
+ 
+    UMLWidget* onWidget(const QPointF& p);
+    UMLWidget* widgetWithID(Uml::ID::Type id);
 
-    virtual void saveToXMI(QDomDocument & qDoc, QDomElement & qElement);
-    virtual bool loadFromXMI(QDomElement & qElement);
+    QSizeF calculateSize(bool withExtensions = true) const;
 
     virtual void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget = 0);
     virtual QPainterPath shape() const;
 
+    virtual void saveToXMI(QDomDocument & qDoc, QDomElement & qElement);
+    virtual bool loadFromXMI(QDomElement & qElement);
+
 public Q_SLOTS:
     virtual void slotMenuSelection(QAction* action);
 
@@ -103,24 +109,26 @@ private Q_SLOTS:
 
 private:
     void updateSignatureTypes();
-    QSize calculateTemplatesBoxSize();
+    QSize calculateTemplatesBoxSize() const;
 
-    QSizeF minimumSize();
+    QSizeF minimumSize() const;
 
     void drawAsCircle(QPainter *p, const QStyleOptionGraphicsItem *option);
-    QSize calculateAsCircleSize();
+    QSize calculateAsCircleSize() const;
 
-    int displayedMembers(UMLObject::ObjectType ot);
+    int displayedMembers(UMLObject::ObjectType ot) const;
     void drawMembers(QPainter *painter, UMLObject::ObjectType ot, Uml::SignatureType::Enum sigType,
                      int x, int y, int fontHeight);
 
     static const int MARGIN;           ///< text width margin
     static const int CIRCLE_SIZE;      ///< size of circle when interface is rendered as such
+    static const int SOCKET_INCREMENT; ///< augmentation of circle for socket (required interface)
 
     VisualProperties   m_visualProperties;
     Uml::SignatureType::Enum m_attributeSignature;   ///< Loaded/saved item.
     Uml::SignatureType::Enum m_operationSignature;   ///< Loaded/saved item.
     AssociationWidget *m_pAssocWidget; ///< related AssociationWidget in case this classifier acts as an association class
+    FloatingTextWidget *m_pInterfaceName;  ///< Separate widget for name in case of interface drawn as circle
 
 };
 
