@@ -68,8 +68,9 @@ public:
     virtual void setMessageText(FloatingTextWidget *ft);
     virtual void setText(FloatingTextWidget *ft, const QString &newText);
 
-    virtual UMLClassifier* seqNumAndOp(QString& seqNum, QString& op);
-    virtual void setSeqNumAndOp(const QString &seqNum, const QString &op);
+    virtual QString lwOperationText();
+    virtual UMLClassifier *lwClassifier();
+    virtual void setOperationText(const QString &op);
 
     virtual void constrainTextPos(qreal &textX, qreal &textY, qreal textWidth, qreal textHeight,
                                   Uml::TextRole::Enum tr);
@@ -129,7 +130,7 @@ public:
     int getMinY();
     int getMaxY();
 
-    qreal onWidget(const QPointF& p);
+    UMLWidget* onWidget(const QPointF& p);
 
     virtual void resizeWidget(qreal newW, qreal newH);
 
