@@ -61,7 +61,9 @@ public:
     void setLocalID(Uml::ID::Type id);
     Uml::ID::Type localID() const;
 
-    virtual QSizeF minimumSize();
+    virtual UMLWidget* widgetWithID(Uml::ID::Type id);
+
+    virtual QSizeF minimumSize() const;
     void setMinimumSize(const QSizeF &size);
 
     virtual QSizeF maximumSize();
@@ -89,8 +91,6 @@ public:
 
     virtual bool activate(IDChangeLog* ChangeLog = 0);
 
-    virtual qreal onWidget(const QPointF &p);
-
     void setPenFromSettings(QPainter &p);
     void setPenFromSettings(QPainter *p);
 
@@ -198,9 +198,11 @@ public:
     virtual void saveToXMI(QDomDocument &qDoc, QDomElement &qElement);
 
     QPointF startMovePosition() const;
+    void setStartMovePosition(const QPointF &position);
+
     QSizeF startResizeSize() const;
 
-    virtual QSizeF calculateSize();
+    virtual QSizeF calculateSize(bool withExtensions = true) const;
     void resize();
 
     bool fixedAspectRatio() const {
@@ -226,7 +228,7 @@ public:
     virtual void setDefaultFontMetrics(QFont &font, UMLWidget::FontType fontType);
     virtual void setDefaultFontMetrics(QFont &font, UMLWidget::FontType fontType, QPainter &painter);
 
-    QFontMetrics &getFontMetrics(UMLWidget::FontType fontType);
+    QFontMetrics &getFontMetrics(UMLWidget::FontType fontType) const;
     void setFontMetrics(UMLWidget::FontType fontType, QFontMetrics fm);
     void setupFontType(QFont &font, UMLWidget::FontType fontType);
 
@@ -241,7 +243,6 @@ public Q_SLOTS:
     virtual void slotTextColorChanged(Uml::ID::Type viewID);
     virtual void slotLineWidthChanged(Uml::ID::Type viewID);
 
-    void slotClearAllSelected();
     void slotSnapToGrid();
 
 signals:
