@@ -26,6 +26,7 @@ class QAction;
 class UMLDoc;
 class UMLObject;
 class UMLScene;
+class UMLWidget;   // required by function onWidget()
 
 /**
  * @short       Common base class for UMLWidget and AssociationWidget
@@ -68,7 +69,8 @@ public:
         wt_Pin,
         wt_ObjectNode,
         wt_Region,
-        wt_Category                 // has UMLObject representation
+        wt_Category,                // has UMLObject representation
+        wt_Port                     // has UMLObject representation
     };
 
     static QString toString(WidgetType wt);
@@ -95,7 +97,7 @@ public:
     void setDocumentation(const QString& doc);
 
     QString name() const;
-    void setName(const QString &strName);
+    virtual void setName(const QString &strName);
 
     QColor lineColor() const;
     virtual void setLineColor(const QColor& color);
@@ -143,6 +145,8 @@ public:
 
     virtual QRectF boundingRect() const;
 
+    virtual UMLWidget* onWidget(const QPointF &p);
+
     virtual void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget = 0);
 
 public Q_SLOTS:
